# ====================================
# = Common options for all platforms =
# ==================================== 

# 0 - release, 1 - 7 debug level
DEBUG = 0

# installation path
PREFIX = /usr/local
# etc dir
ETCDIR = $(PREFIX)/etc/$(PROGLNAME)
# where put reports (you can change it later via free-sa.conf configuration file)
WWWDIR = $(PREFIX)/var/www/htdocs/$(PROGLNAME)
# dir for temporary data storing (you can change it later via free-sa.conf configuration file)
TMPDIR = /var/cache/$(PROGLNAME)
# where localisation files to put (it is safe to leave default value unchanged)
LOCDIR = $(PREFIX)/share/$(PROGLNAME)
# where log file lives (you can change it later via free-sa.conf configuration file)
LOGFILE = /var/log/squid/access.log

# C compiler (prefer native system compiler for its perfomance and code optimizations)
CC = cc
# Library archiver
AR = ar
# Sort utility (define it to /bin/sort for RedHat/Fedora/CentOS Linux)
SORT = /usr/bin/sort

# Compiler flags
SACFLAGS = 
# Linker flags (required libraries)
SALDFLAGS = -lm
# ADDITIONAL linker flags to SALDFLAGS for free-sa.cgi
#SACGILDFLAGS = -static

# Install program
# (set to SAinstall.sh full path from distribution, if your install is not GNU-like)
INSTALL = install

# Full large file support, gives little perfomance degrade (undefine to disable)
LARGEFILE = yes

# ==============================================
# = GNU C compiler and linker specific options =
# ==============================================

# CPU (allowed by gcc '-march' option: i486, pentium3, athlon, nocona, k8 ...)
GCCARCH = i486

# GNU compiler flags (by line number):
#  1  Optimizations
#  2  Standard compliance checking
# 3-7 Warnings flags (recommended by AMD)
#ADDCFLAGS = \
#	-O4 -pipe -march=$(GCCARCH) -fomit-frame-pointer			\
#	-std=c99								\
#	-W -Wall -Wextra -Wredundant-decls -Wnested-externs -Wstrict-prototypes	\
#	-Wmissing-prototypes -Wpointer-arith -Winline -Wcast-qual		\
#	-Wreturn-type -Wswitch -Wshadow -Wcast-align -Wuninitialized		\
#	-Wchar-subscripts -Wuninitialized -Wbad-function-cast -Wwrite-strings

# GNU linker flags
#ADDLDFLAGS = --relax -mrelax

# =================================================
# = HP-UX C compiler and linker specific options  =
# =================================================

# CPU (allowed by cc '+DA' option: 1.1, 867, 2.0, 2.0W, portable)
HCCARCH = 2.0W

# HP-UX compiler flags (by line number):
# 1 Optimization, data model, const handling
# 2 Standard compliance checking
# 3 Warnings flags (maximal)
#ADDCFLAGS = +DA$(HCCARCH) +O4 +DD64 +ESconstlit \
#	-AC99 \
#	+w1 +M2 

# ======================
# = DO NOT EDIT BELOW! =
# ======================

PROGNAME = Free-SA
PROGVERSION = 1.4.3
PROGURL = http://free-sa.sourceforge.net
PROGLNAME = free-sa

DEFS = -DDEBUG=$(DEBUG) \
       -DPREFIX=\"$(PREFIX)\" \
       -DETCDIR=\"$(ETCDIR)\" -DWWWDIR=\"$(WWWDIR)\" -DTMPDIR=\"$(TMPDIR)\" -DLOCDIR=\"$(LOCDIR)\" \
       -DLOGFILE=\"$(LOGFILE)\" \
       -DLARGEFILE=\"$(LARGEFILE)\" \
       -DCMDSORT=\"$(SORT)\" \
       -DPROGNAME=\"$(PROGNAME)\" -DPROGVERSION=\"$(PROGVERSION)\" -DPROGURL=\"$(PROGURL)\" -DPROGLNAME=\"$(PROGLNAME)\"

# All compiler flags
CFLAGS = $(SACFLAGS) $(ADDCFLAGS) -I. -I.. $(DEFS)

# All linker flags
LDFLAGS = $(SALDFLAGS) $(ADDLDFLAGS) $(CFLAGS)

# Directory function
MAKEDIRS = @for SDIR in $(SUBDIRS); do cd $$SDIR && $(MAKE) $@; cd ..; done

# Install variations
INSTALLFIL = $(INSTALL) -m 0644 -v
INSTALLBIN = $(INSTALL) -m 0755 -v
INSTALLDIR = $(INSTALL) -m 0755 -v -d
INSTALLODR = $(INSTALL) -m 0700 -v -d

# All text files variable->value replacer
SEDREPLACE = sed -e 's|LOGFILE|$(LOGFILE)|g'   -e 's|TMPDIR|$(TMPDIR)|g'           -e 's|ETCDIR|$(ETCDIR)|g' \
                 -e 's|WWWDIR|$(WWWDIR)|g'     -e 's|PREFIX|$(PREFIX)|g'                                     \
                 -e 's|PROGNAME|$(PROGNAME)|g' -e 's|PROGVERSION|$(PROGVERSION)|g'                           \
                 -e 's|PROGURL|$(PROGURL)|g'   -e 's|PROGLNAME|$(PROGLNAME)|g'
