#include "global.h"


struct SAentry *Users;
char rurl[URLSIZE];
struct SAB3v SAB3variants[6] = {
    { 999999999999999.0,    "P", 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 },
    {    999999999999.0,    "T",          1024.0 * 1024.0 * 1024.0 * 1024.0 },
    {       999999999.0,    "G",                   1024.0 * 1024.0 * 1024.0 },
    {          999999.0,    "M",                            1024.0 * 1024.0 },
    {             999.0,    "k",                                     1024.0 },
    {               0.0, "\000",                                        1.0 }
};


int compSAentry(const struct SAentry *el1, const struct SAentry *el2) {
    return strcmp(el1->name, el2->name);
}


int compSAuconv(const struct SAuconv *el1, const struct SAuconv *el2) {
    return strcmp(el1->iname, el2->iname);
}


int rcompSAentryB(const struct SAentry *el1, const struct SAentry *el2) {
    return SAnumcmp(el1->Bytes, el2->Bytes);
}


int compSI(const short int *el1, const short int *el2) {
    return SAnumcmp(el1[0], el2[0]);
}


struct SAentry *SAentryByName(const char *sname) {
    struct SAentry key;
    
    key.name = sname;
    return (struct SAentry *)bsearch(&key, Users, (size_t)icfg.ueCount, sizeof(struct SAentry), (int (*)(const void *, const void *))compSAentry);
}


const char *LimitedURL(const char *lurl) {
    size_t slen, halful;
    
    if (!cfg.URLLimit || (slen = strlen(lurl)) <= (size_t)cfg.URLLimit) return lurl;
	
    halful = (size_t)cfg.URLLimit / 2L;
	    
    strncpy(rurl, lurl, halful - 2L);
    rurl[halful - 2L] = rurl[halful - 1L] = rurl[halful] = '.';
    rurl[halful + 1L] = '\0';
    return strcat(rurl, &lurl[slen - halful + 1L]);
}


char *FixFileName(char *tfname) {
    char *t = tfname;
    if (cfg.UserUnescape) SAunescape(tfname);
    for (; t[0] != '\0'; t++) 
	if (t[0] == '\\' || t[0] == '%' || t[0] == '$' || t[0] == '/' || t[0] == '`' || t[0] == '|' || t[0] == '&') t[0] = '_';
    return tfname;
}


char *SAB0(double ibytes) {
    static char retval[32];
        
    SAsnprintf(retval, sizeof(retval), "%'.0f", ibytes);
            
    return retval;
}


char *SAB1(double ibytes) {
    static char retval[32];
    
    SAsnprintf(retval, sizeof(retval), "%'.0f%s", round(ibytes / icfg.Divisor), icfg.locDivisor);
    
    return retval;
}


char *SAB2(double ibytes) {
    static char retval[32];
    
    SAsnprintf(retval, sizeof(retval), "%'.2f%s", ibytes / icfg.Divisor, icfg.locDivisor);
    
    return retval;
}


char *SAB3(double ibytes) {
    static char retval[6];
    short int i = 0;

    while(i < 6 && ibytes <= SAB3variants[i].vlimit) i++;
    SAsnprintf(retval, sizeof(retval), "%.0f%s", round(ibytes / SAB3variants[i].divisor), SAB3variants[i].munit);
    
    return retval;
}


char *GetDT(time_t idt, const char *fmt) {
    static char tdt[DATETIMESIZE];

    if (!strftime(tdt, DATETIMESIZE - 1L, fmt, localtime(&idt))) SAexit("Can't convert datetime %ld", idt);

#if DEBUG>=4
    SAdebug("%d->%s", idt, tdt);
#endif
    
    return tdt;
}


char *GetD(struct tm *itm, const char *fmt) {
    static char tdt[DATETIMESIZE];

    if (!strftime(tdt, DATETIMESIZE - 1L, fmt, itm)) SAexit("Can't convert date %d-%d-%d", itm->tm_year, itm->tm_mon, itm->tm_mday);

#if DEBUG>=4
    SAdebug("%d-%d-%d->%s", itm->tm_year, itm->tm_mon, itm->tm_mday, tdt);
#endif
    
    return tdt;
}


time_t GetDZT(const char *idate) {
    struct tm dtconv;

    if (strptime(idate, "%x", &dtconv) == NULL) SAexit("Can't convert date %s", idate);
    dtconv.tm_sec = dtconv.tm_min = dtconv.tm_hour = 0;
    
#if DEBUG>=4
    SAdebug("%s->%s", idate, GetDT(mktime(&dtconv), "%x %X"));
#endif

    return mktime(&dtconv);
}


time_t GetDW(const char *iperiod) {
    if (!strcmp(iperiod, "hour"))         return SAHOUR;
    else if (!strcmp(iperiod, "day"))     return SADAY;
    else if (!strcmp(iperiod, "week"))    return SAWEEK;
    else if (!strcmp(iperiod, "month"))   return SAMONTH;
    else if (!strcmp(iperiod, "quarter")) return SAQUARTER;
    else if (!strcmp(iperiod, "year"))    return SAYEAR;
    else return 0L;
}


/* Compute period and replace specified at CL start/end dates with real ones */
void ComputePeriod(time_t rsdate, time_t redate) {
    time_t timediff;
    struct tm *ttm;

    /* Period work, part1: real start date */
    icfg.SDate = rsdate;
    if ((ttm = localtime(&icfg.SDate)) == NULL) SAexit("Can't convert calendar time: %ld", icfg.SDate);
    icfg.SYear = ttm->tm_year;
    strcpy(icfg.Period, GetDT(icfg.SDate, "%x-"));
    
    /* Period work, part2: real end date */
    if (redate < icfg.EDate) icfg.EDate = redate;
    if ((ttm = localtime(&icfg.EDate)) == NULL) SAexit("Can't convert calendar time: %ld", icfg.EDate);
    icfg.EYear = ttm->tm_year;
    strcat(icfg.Period, GetDT(icfg.EDate, "%x"));
    
    timediff = icfg.EDate - icfg.SDate;
    if (timediff <= SADAY) icfg.PScale = 24;
    else if (timediff <= SAMONTH) icfg.PScale = 31;
    else icfg.PScale = 12;
}


/* for url relative indicators */
void Indics(FILE *iFile, short int iIndic) {
    /* assume that indicators will be writed only to end of table row */
    if (cfg.Indics && iIndic) {
	if (SAgetbit(iIndic, PAF)) SAfwrite1((const char *)"<td class=paf>", 14L, iFile);
	if (SAgetbit(iIndic, SAF)) SAfwrite1((const char *)"<td class=saf>", 14L, iFile);
	if (SAgetbit(iIndic, PDN)) SAfwrite1((const char *)"<td class=pdn>", 14L, iFile);
	if (SAgetbit(iIndic, SDN)) SAfwrite1((const char *)"<td class=sdn>", 14L, iFile);
	if (SAgetbit(iIndic, CCT)) SAfwrite1((const char *)"<td class=cct>", 14L, iFile);
	if (SAgetbit(iIndic, PST)) SAfwrite1((const char *)"<td class=pst>", 14L, iFile);
	if (SAgetbit(iIndic, DLD)) SAfwrite1((const char *)"<td class=dld>", 14L, iFile);
	if (SAgetbit(iIndic, LCF)) SAfwrite1((const char *)"<td class=lcf>", 14L, iFile);
    }
    SAfwrite1((const char *)"\n", 1L, iFile);
}


/* for user relative indicators */
void IndicsByIndex(FILE *iFile, int i) {
    /* assume that indicators will be writed only to end of table row */
    if (cfg.Indics) {
	if (Users[i].TTcount[PAF]) SAfwrite1((const char *)"<td class=paf>", 14L, iFile);
	if (Users[i].TTcount[SAF]) SAfwrite1((const char *)"<td class=saf>", 14L, iFile);
	if (Users[i].TTcount[PDN]) SAfwrite1((const char *)"<td class=pdn>", 14L, iFile);
	if (Users[i].TTcount[SDN]) SAfwrite1((const char *)"<td class=sdn>", 14L, iFile);
	if (Users[i].TTcount[CCT]) SAfwrite1((const char *)"<td class=cct>", 14L, iFile);
	if (Users[i].TTcount[PST]) SAfwrite1((const char *)"<td class=pst>", 14L, iFile);
	if (Users[i].TTcount[DLD]) SAfwrite1((const char *)"<td class=dld>", 14L, iFile);
	if (Users[i].TTcount[LCF]) SAfwrite1((const char *)"<td class=lcf>", 14L, iFile);
    }
    SAfwrite1((const char *)"\n", 1L, iFile);
}


void SAunescape(char *istring) {
    size_t slen;
    char *pstring, *nstring, *retval;
    char hexchar[3] = "  \000";
    
    slen = strlen(istring);
    retval = nstring = istring;
    
    for (pstring = istring; pstring != istring + slen + 1L; pstring++) {
	if (pstring[0] == '%' && SAisxdigit(pstring[1]) && SAisxdigit(pstring[2]) && pstring <= istring + slen - 2L) {
		hexchar[0] = pstring[1];
		hexchar[1] = pstring[2];
		
		nstring++[0] = (char)strtol(hexchar, NULL, 16);
		pstring += 2;
	} else nstring++[0] = pstring[0];
    }
}
