#include "global.h"


double ccBytes = 0, allmBytes[5] = { 0, 0, 0, 0, 0 }, allmSeconds[5] = { 0, 0, 0, 0, 0 };
int ccCount = 0;
struct BSC ec[CODESSIZE];
struct BSC all[5] = { { 0.0, 0.0, 0 }, { 0.0, 0.0, 0 }, { 0.0, 0.0, 0 }, { 0.0, 0.0, 0 }, { 0.0, 0.0, 0 } };


char FilterI(const char *iEntry, const void **iList, const char iMode, const short int iCount);
char FilterI(const char *iEntry, const void **iList, const char iMode, const short int iCount) {
    short int i;


    switch (iMode) {
	case 0:
	case 1:
		for (i = 0; i < iCount; i++) if (strstr(iEntry, (const char *)iList[i]) != NULL) return iMode%2;
		break;
	case 2:
	case 3:
		if (bsearch(&iEntry, iList, (size_t)iCount, sizeof(char *), (int (*)(const void *, const void *))SApstrcmp) != NULL) return iMode%2;
		break;
	case 4:
	case 5:
		for (i = 0; i < iCount; i++) if (!regexec(&(((regex_t *)iList)[i]), iEntry, 0L, NULL, 0)) return iMode%2;
		break;
    }

    return !(iMode%2);
}


/* returns 0 if record must be skipped (it's in exclude list or not in existing include list) */
char Filter(const struct FConfig *fcfg, const char *turl, const char *iname, const short int tcode, const double tbytes, const char tstat, const char tmethod);
char Filter(const struct FConfig *fcfg, const char *turl, const char *iname, const short int tcode, const double tbytes, const char tstat, const char tmethod) {
    if (fcfg->stats != '\0') {
	if (SAgetbit(fcfg->stats, tstat)) {
	    if (fcfg->statMode%2) return 0;
	} else if (!(fcfg->statMode%2)) return 0;
    }

    if (fcfg->bytes) {
	if (tbytes > fcfg->bytes) {
	    if (fcfg->bytesMode%2) return 0;
	} else if (!(fcfg->bytesMode%2)) return 0;
    }
    
    if (fcfg->methods != NULL) {
	if (strchr(fcfg->methods, (int)tmethod) != NULL) {
	    if (fcfg->methodMode%2) return 0;
	} else if (!(fcfg->methodMode%2)) return 0;
    }
    
    if (fcfg->codeCount) {
	if ((bsearch(&tcode, fcfg->codes, (size_t)fcfg->codeCount, sizeof(short int), (int (*)(const void *, const void *))compSI)) != NULL) {
	    if (fcfg->codeMode%2) return 0;
	} else if (!(fcfg->codeMode%2)) return 0;
    }
    
    if (fcfg->inameCount) if (FilterI(iname, (const void **)fcfg->inames, fcfg->inameMode, fcfg->inameCount)) return 0;
    if (fcfg->urlCount) if (FilterI(turl, (const void **)fcfg->urls, fcfg->urlMode, fcfg->urlCount)) return 0;
    
    return 1;
}


short int ComputeIndics(short int tcode, char tmethod, char *turl, char *tstat, char *iname, double tbytes) {
    short int tIndic = 0;
    int i;
    char *t;


    /* saf, paf, sdn, pdn */
    switch (LogFormats[cfg.LogFormat].Class) {
        case PROXY:
        case WWW:
    	    switch (tcode) {
		case 401: 
			if (cfg.TTR[SAF]) SAsetbit(tIndic, SAF);
			break;
		case 403:
			if (cfg.TTR[PDN] && *tstat == 1) SAsetbit(tIndic, PDN);
			else if (cfg.TTR[SDN]) SAsetbit(tIndic, SDN);
			break;
		case 407:
			if (cfg.TTR[PAF]) SAsetbit(tIndic, PAF);
		default:
			if (*tstat == 1) *tstat = 3; /* PER: replace not proxy denied by other local */
			break;
	    }
	    break;
	    
	case MAIL:
	    switch (tcode) {
		case 451:
			if (cfg.TTR[SDN]) SAsetbit(tIndic, SDN);
			break;
		default:	
			if (cfg.TTR[PDN] && *tstat == 1) SAsetbit(tIndic, PDN);
			break;
	    }
	    break;
	}
	
    /* cct, pst */
    switch(tmethod) {
        case 'C': if (cfg.TTR[CCT]) SAsetbit(tIndic, CCT); break;
        case 'P': if (cfg.TTR[PST]) SAsetbit(tIndic, PST); break;
    }
	
    /* dld */
    if (cfg.TTR[DLD] && tmethod != 'C' && tbytes >= cfg.DLDMin)
        for (i = 0; i < icfg.dldCount; i++) {
    	    t = &turl[strlen(turl) - strlen(icfg.Downloads[i])];
	    
	    if (!strcmp(t, icfg.Downloads[i])) {
	        SAsetbit(tIndic, DLD);
	        break;
	    }
	}

    /* lcf */
    if (cfg.TTR[LCF]) if (Filter(&lfcfg, turl, iname, tcode, tbytes, *tstat, tmethod)) SAsetbit(tIndic, LCF);

    return tIndic;
}


struct SAentry *cures = NULL;
struct SAentry *ComputeUser(char *iname, char *tip) {
    struct hostent *hp;
    struct in_addr resip;
    struct SAuconv ukey, *ures;
    
    
    /* Speed up: binary search is many strcmp's, we do only two first */
    if (cures != NULL) {
	if (!strcmp(iname, cures->name)) {
	    if (iname == tip && cures->name == cures->ip) return cures;
	    if (!strcmp(tip, cures->ip)) return cures;
	}
	cures = SAentryByName(iname);
    }
    
    /* If internal name not found in Users or Users is empty (i.e. 1st iteration) */
    if (cures == NULL) {
	Users = (struct SAentry *)SArealloc((void *)Users, sizeof(struct SAentry) * (icfg.ueCount + 1L));
	cures = &Users[icfg.ueCount];
	icfg.ueCount++;
	/* Fill user unsorted file name u_USERNAME.uns if not called from CGI */
	if (cfg.UR || cfg.TSR) {
	    SAsnprintf(cures->fname, sizeof(cures->fname), TEMPL_U_UNS, cfg.TmpDir, iname);
	    cures->file = SAfopenI("w", cures->fname);
	}
	cures->ip = SAstrdup(tip);
	/* Clear (i.e. set to 0) starting from vname up to the end of structure */
	memset((void *)&cures->vname, 0, sizeof(cures->vname) + sizeof(cures->Bytes) + sizeof(cures->count) + sizeof(cures->TTcount));
	
	/* cures->name is saved internal name */
	if (iname != tip) cures->name = SAstrdup(iname);
	else {
	    /* If we use IP instead of name and should resolve it then try to fill vuser */
	    if (cfg.ResolveIP && inet_pton(AF_INET, iname, &resip) != 0) 
		if ((hp = gethostbyaddr((char *)&resip.s_addr, sizeof(in_addr_t), AF_INET)) != NULL) cures->vname = SAstrdup(hp->h_name);
	    cures->name = cures->ip;
	}
	
	/* If UserTab have entries and we not resolved IP before, then fill vname according it */
	if (icfg.utCount && !cures->vname) {
	    ukey.iname = iname;
	    ures = (struct SAuconv *)bsearch(&ukey, icfg.Usertab, (size_t)icfg.utCount, sizeof(struct SAuconv), (int (*)(const void *, const void *))compSAuconv);
	    if (ures != NULL) cures->vname = ures->ename;
	}
	    
	/* If we still not filled vname, then fill it as name */
	if (!cures->vname) cures->vname = cures->name;
	
	qsort(Users, (size_t)icfg.ueCount, sizeof(struct SAentry), (int (*)(const void *, const void *))compSAentry);

	/* After sorting we should recover pointer (iname points to tname, so it's not damaged by qsort) */
	cures = SAentryByName(iname);
    }
    
    return cures;
}


/* Function initial/finish log position: any, but meaningful/any */
void CollectLog(FILE *Log, time_t *rsdate, time_t *redate) {
    char tip[IPSIZE], tname[USERNAMESIZE], tmethod, turl[URLSIZE], tstat;
    double tBytes, tSeconds;
    short int tcode, tIndic;
    struct SAentry *res;
    char *t = NULL, *iname;
    time_t ttime, mindate = 0L;
    int i;
    
    
    while(READ_RECORD(Log, &ttime, &tSeconds, tip, &tcode, &tBytes, &tmethod, turl, tname, &tstat)) {
	/* Check if record in specified date range (according to linearity of log */
	if (LogFormats[cfg.LogFormat].isLinear) { if (ttime > icfg.EDate) break; }
	else if (ttime < icfg.SDate || ttime > icfg.EDate) continue;
	
        /* Internal name resolution and fix */
	if (!cfg.INameUser || tname[0] == '-') iname = tip;
	else iname = FixFileName(tname);

	/* Apply global filter */
	if (icfg.GF) if (!Filter(&gfcfg, turl, iname, tcode, tBytes, tstat, tmethod)) continue;
	
	/* Fill start date for period computing */
	if (!icfg.ueCount) mindate = ttime;
	
	/* Search for internal name in Users */
	res = ComputeUser(iname, tip);
	
	tIndic = ComputeIndics(tcode, tmethod, turl, &tstat, iname, tBytes);
	
	/* TT_UNS: write to file & user TT-stats */
	for (i = 0; i < TTNUMBER; i++)
	    if (SAgetbit(tIndic, i)) {
		/* write only if limit allows it */
		if (!cfg.ttUserLimit[i] || res->TTcount[i] < cfg.ttUserLimit[i]) {
		    if (i >= CCT) SAfprintf(TT[i].file, "%s %ld %s %.0f %hd\n", iname, ttime, turl, tBytes, tIndic);
		    else SAfprintf(TT[i].file, "%s %ld %s %hd\n", iname, ttime, turl, tIndic);
		    TT[i].count++;
		}
		res->TTcount[i]++;
	    }
	
	/* PER: collect */
	if (cfg.PER) {
	    if (tstat) {
		i = (int)tstat;
		all[i].Bytes += tBytes;
		all[i].Seconds += tSeconds;
	        all[i].Count++;
		
		if (tBytes > allmBytes[i]) allmBytes[i] = tBytes;
		if (tSeconds > allmSeconds[i]) allmSeconds[i] = tSeconds;
	    } else {
		if (tBytes > allmBytes[4]) allmBytes[4] = tBytes;
		if (tSeconds > allmSeconds[4]) allmSeconds[4] = tSeconds;
	    }
	    
	    if (tcode >= CODESSIZE) SAexit("Unknown code: %hd, please contact developers", tcode);
	    ec[tcode].Bytes += tBytes;
	    ec[tcode].Seconds += tSeconds;
	    ec[tcode].Count++;
	}
	
	/* Compute user stats and write to user file */
	if (cfg.FullTraffic || !tstat) {
	    /* Compute user stats */
	    res->Bytes += tBytes;
	    res->count++;
	    
	    /* Compute total stats for TopUser report */
	    ccBytes += tBytes;
	    ccCount++;
	    
	    if (cfg.UR || cfg.TSR) {
		/* Shorten URL (we jump over "http://" or "ftp://" here) */
		if (!cfg.FullURL && tmethod != 'C') {
		    for (t = &turl[7]; t[0] != '\0' && t[0] != '/'; t++);
		    t[0] = '\0';
		}
	    
		SAfprintf(res->file, "%.0f %ld %s %hd %c\n", tBytes, ttime, turl, tIndic, tmethod);
	    }
	}
	
	/* Compute total stats */
	all[0].Bytes += tBytes;
	all[0].Seconds += tSeconds;
	all[0].Count++;
    }

    /* Exit if no records found */
    if (!icfg.ueCount) SAexit("No records found");

    /* Close log file */
    SAfclose(Log, cfg.Log);
    
    /* Fill real start/end dates */
    *rsdate = mindate;
    *redate = ttime;
}


/* Function initial/finish log position: any/requested */
char SeekLog(FILE *Log, time_t sdate) {
    SAOFF_T rseek = 0L, sseek = 0L, eseek = 0L;
    time_t ttime, mindate, maxdate;


    /* Rewind first, as we not know current position and alignment */
    rewind(Log);
    /* Seek in linear log */
    if (LogFormats[cfg.LogFormat].isLinear) {
	/* Fast check for first and last records dates */
	ShowLogI(Log, &mindate, &maxdate);
	if (mindate >= sdate) {
	    rewind(Log);
	    return 0;
	}
	if (maxdate < sdate) return 1;

	/* Save EOF position as end of 1st range (because ShowLogI seeks to EOF) */
	eseek = SAftell(Log);
	
	/* Fast (?) binary search */
	for (;;) {
	    /* Break if range is little enough to process it sequentally 
	       (because logs may have records with same time) */
	    if ((eseek - sseek) <= (SAOFF_T)RECORDSIZE) break;
	    /* Seek to middle of range */
	    SAfseek(Log, sseek + (eseek - sseek) / 2L, SEEK_SET, cfg.Log);
	    /* Align to next record start */
	    READ_ALIGN(Log);
	    /* Save real record start position */
	    rseek = SAftell(Log);
	    /* Read record time */
	    READ_DT(Log, &ttime);
#if DEBUG>=1
	    SAdebug("Range %lld <-> %lld, %ld %s %ld position=%lld", (long long int)sseek, (long long int)eseek, ttime, (ttime >= sdate) ? ">=" : "< ", sdate, (long long int)rseek);
#endif
	    if (ttime >= sdate) eseek = rseek;
	    else sseek = rseek;
	}
#if DEBUG>=1
	SAdebug("Pre-final range %lld <-> %lld", (long long int)sseek, (long long int)eseek);
#endif
	/* Find position with requested date sequentally */
	READ_ALIGN(Log);
	do {
	    rseek = SAftell(Log);
	    if (!READ_DT(Log, &ttime)) SAexit("Error seeking record");
	} while (ttime < sdate);
	/* Recover position */
	SAfseek(Log, rseek, SEEK_SET, cfg.Log);
#if DEBUG>=1
	SAdebug("Final position found: %lld", (long long int)rseek);
#endif
    } else SAwarning("Non-linear log format detected, can't seek for fast data processing");
    
    return 0;
}


/* Function initial/finish log position: start/end */
void ShowLogI(FILE *Log, time_t *mindate, time_t *maxdate) {
    time_t ttime;
    
    
    *mindate = SAMAXTIMET;
    *maxdate = 0L;
    
    if (LogFormats[cfg.LogFormat].isLinear) {
	/* Read first record date */
	if (!READ_DT(Log, &ttime)) SAexit("Can't read log");
	*mindate = ttime;
	
	/* For next function call we need log file size */
	SAfseek(Log, 0L, SEEK_END, cfg.Log);
	
	/* If log is not short we rewind it to prelast records */
	if (SAftell(Log) >= (SAOFF_T)RECORDSIZE) SAfseek(Log, -((SAOFF_T)RECORDSIZE), SEEK_END, cfg.Log);
	else rewind(Log);
	
	READ_ALIGN(Log);
	while(READ_DT(Log, &ttime));
	*maxdate = ttime;
    } else while(READ_DT(Log, &ttime)) {
	if (ttime < *mindate) *mindate = ttime;
	if (ttime > *maxdate) *maxdate = ttime;
    }
}


/* Function initial/finish log position: start/end */
void ShowLog(FILE *Log) {
    time_t mindate, maxdate;

    ShowLogI(Log, &mindate, &maxdate);
    SAshow("Log file information:\n\
File name:\t\t%s\n\
File size:\t\t%'lld\n\
Start date and time:\t%s\n\
",
    cfg.Log, (long long int)SAftell(Log), GetDT(mindate, "%x %X"));
    /* Because GetDT is not volatile */
    SAshow("End date and time:\t%s\n", GetDT(maxdate, "%x %X"));
}


/* Function initial/finish log position: start/log is closed within this function */
void CropLog(FILE *Log) {
    time_t ttime;
    FILE *tLog;
    char *tbuf, tfname[FILENAMESIZE];

    
    SAsnprintf(tfname, sizeof(tfname), "%s.new", cfg.Log);
    tLog = SAfopenI("w", tfname);
    
    tbuf = (char *)SAmalloc(RECORDSIZE);
    while (READ_FULLDT(Log, &ttime, tbuf)) {
	/* Skip records in specified date range */
	if (ttime >= icfg.SDate && ttime <= icfg.EDate) continue;
	
	SAfwrite1(tbuf, strlen(tbuf), tLog);
	SAfwrite1("\n", 1L, tLog);
    }
    free(tbuf);
    
    SAfclose(tLog, tfname);
    SAfcloseU(Log, cfg.Log);
    if (rename(tfname, cfg.Log)) SAexit("Can't rename %s to %s. %s", tfname, cfg.Log, strerror(errno));
}


/* Function initial/finish log position: start/start */
FILE *InitLog(void) {
    FILE *retval;
    short int i, j, tcode, checkval;
    time_t ttime;
    double tSeconds, tBytes;
    char tip[IPSIZE], tname[USERNAMESIZE], tmethod, turl[URLSIZE], tstat;
    

    retval = SAfopenI("r", cfg.Log);

    /* Start autodetection of log format if it's not specified */
    if (cfg.LogFormat == -1)
	for (i = 0; (size_t)i < sizeof(LogFormats)/sizeof(struct LogFormat); i++) {
	    READ_RECORDI = LogFormats[i].ReadRecordI;
	    READ_RECORD = LogFormats[i].ReadRecord;


#if DEBUG>=1
	    SAdebug("Checking for %s", LogFormats[i].name);
#endif
	
	    /* Try to read cfg.AutoRecords number of records using log format read function */
	    for (checkval = j = 0; j < cfg.AutoRecords; j++) checkval += READ_RECORD(retval, &ttime, &tSeconds, tip, &tcode, &tBytes, &tmethod, turl, tname, &tstat);

	    /* Rewind to start of log for next try or processing */
	    rewind(retval);

	    if (checkval == cfg.AutoRecords) {
#if DEBUG>=1
		SAdebug("Detected %s with %s records order", LogFormats[i].name, LogFormats[i].isLinear ? "linear" : "random");
#endif
		cfg.LogFormat = i;
		break;
	    }
#if DEBUG>=1
	    else SAdebug("NOT %s (%hd/%hd readed)", LogFormats[i].name, checkval, cfg.AutoRecords);
#endif
	}
    
    if (cfg.LogFormat == -1) SAexit("Log format autodetection failed");

    /* Clear mail records buffers */
    if (LogFormats[cfg.LogFormat].Class == MAIL) DelAllMR();
    
    /* Set READ_RECORD for manually specified log format (others - for all) */
    READ_RECORDI = LogFormats[cfg.LogFormat].ReadRecordI;
    READ_RECORD = LogFormats[cfg.LogFormat].ReadRecord;
    READ_DT = LogFormats[cfg.LogFormat].ReadDT;
    READ_FULLDT = LogFormats[cfg.LogFormat].ReadFullDT;
    READ_ALIGN = LogFormats[cfg.LogFormat].ReadAlign;
    
    /* Disable unsupported by log format reports: TT & TSR */
    for (j = 0; j < 6; j++) if (cfg.TTR[j] == 1 && LogFormats[cfg.LogFormat].Capabilities[j] == 0) cfg.TTR[j] = 0;
    if (LogFormats[cfg.LogFormat].Capabilities[7] == 0) cfg.TSR = 0;
    
    
    return retval;
}
