#include "global.h"


int pval;


int ErrOut(const char Mode, const char *func, const char *fmt, ...) {
    FILE *errout;
    va_list args;
    
    
    if (icfg.Mode == 5) {
	errout = stdout;
	SAfwrite1("<!-- ", 5L, errout);
    } else errout = stderr;
    
    switch(Mode) {
	case 2: SAfwrite1("SAWARNING", 9L, errout); break;
	case 1: SAfwrite1("SADEBUG", 7L, errout); break;
	case 0: SAfwrite1("SAERROR", 7L, errout); break;
    }
    
    SAfprintf(errout, ": [%s] ", func);
    va_start(args, fmt);
    SAvfprintf(errout, fmt, args);
    va_end(args);
    
    if (icfg.Mode == 5) SAfwrite1(" -->\n", 5L, errout);
    else SAfwrite1("\n", 1L, stderr);
    fflush(errout);
    
    if (!Mode) exit(EXIT_FAILURE);
    
    return 0;
}


void SAshow(const char *fmt, ...) {
    va_list args;
    
    va_start(args, fmt);
    SAvfprintf(stdout, fmt, args);
    va_end(args);
}


void SAsystem(const char *program, ...) {
    pid_t tpid;
    int tmp = 0, status;
    va_list args;
    char *argsarray[ARGSCOUNT];

    tpid = fork();
    if (tpid == 0) {
	va_start(args, program);
	
	for (argsarray[0] = SAstrdup(program); (argsarray[tmp] = va_arg(args, char *)) != NULL; tmp++);

#if DEBUG >= 2
	SAdebug ("Executing '%s' with next arguments:", program);
	for (tmp = 0; argsarray[tmp] != NULL; tmp++) {
	    SAdebug ("%d. '%s'", tmp, argsarray[tmp]);
	}
#endif

	execv(program, argsarray);
	SAexit("Can't execute %s", program);
    }

    if (waitpid(tpid, &status, 0) < 0) SAexit("waitpid %s, %s", program, strerror(errno));;
    if (WIFSIGNALED(status)) SAexit("%s got signal", program);
    if (WIFEXITED(status)) if (!WEXITSTATUS(status)) return;
    
    /* Some error during execution occured */
    SAexit("%s returned %d", program, WEXITSTATUS(status));
}


void *SAmalloc(size_t size) {
    void *retval;
    
    retval = malloc(size);
    if (retval == NULL) SAexit("No free memory available");
    
    return retval;
}


void *SArealloc(void *ptr, size_t size) {
    void *retval;
    
    retval = realloc(ptr, size);
    if (retval == NULL) SAexit("No free memory available");
    
    return retval;
}


char *SAstrdup(const char *src) {
    char *retval;
    if ((retval = strdup(src)) == NULL) SAexit("No free memory available");
    return retval;
}


int SApstrcmp(const char **el1, const char **el2) {
    return strcmp(*el1, *el2);
}
